/***************************************************************************
 *   Copyright (C) 2004 by Cyril Bosselut                                  *
 *   bosselut@b1project.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ktagmainwin.h"

KTagMainWin::KTagMainWin( QWidget* parent, const char* name, WFlags fl ) : DCOPObject( "KTag" ), KMainWindow( parent, name, fl ){
	setupActions();
	setAcceptDrops(TRUE);
	tabWidget = new KTabWidget( this, "tabWidget" );
	tabWidget->setEnabled( true );
	tabWidget->setHoverCloseButton( TRUE );
	tabWidget->setAutoMask( TRUE );
	tabWidget->setAcceptDrops(TRUE);
	setCentralWidget(tabWidget);
	connect(qApp,SIGNAL(lastWindowClosed()),qApp,SLOT(quit()));
	connect(tabWidget, SIGNAL(closeRequest(QWidget *)), this, SLOT(closeTab(QWidget *)));
}

KTagMainWin::~KTagMainWin(){
}

void KTagMainWin::setupActions(){
	KStdAction::save(this, SLOT(saveFile()), actionCollection());
	KStdAction::preferences(this, SLOT(config()), actionCollection());
	KStdAction::open(this, SLOT(chooseFile()), actionCollection());
	(void) new KAction(tr2i18n("Save &All"), 0, this, SLOT(saveAll()), actionCollection(), "saveAll");
	createGUI();
}

void KTagMainWin::saveFile(){
	kdDebug("MainWin") << "Saving current file" << endl;
	Ktag* mKtag = ktagList.at(tabWidget->currentPageIndex());
	mKtag->writeTags(false);
	kdDebug("MainWin") << mKtag->mAudioFile << endl;
}

void KTagMainWin::saveAll(){
	if(tabWidget->count() > 0){
		KProgressDialog *p = new KProgressDialog(this, "progress", tr2i18n("Writing tags"), tr2i18n("Saving all files"), true);
		p->showCancelButton(false);
		p->progressBar()->setTotalSteps(tabWidget->count());
		kdDebug("MainWin") << "Saving all tabs " << endl;
		for(int i = 0; i <= tabWidget->count()-1; i++){
			if(ktagList.at(i)->modified){
				ktagList.at(i)->writeTags(true);
			}
			p->progressBar()->setProgress(i+1);
		}
	}
}

void KTagMainWin::closeEvent(QCloseEvent *ce){
	if(tabWidget->count() > 0){
		kdDebug("MainWin") << "Closing tabs " << endl;
		while(tabWidget->count() > 0){
			closeTab(tabWidget->page(0));
		}
	}
	ce->accept();
}

bool KTagMainWin::chooseFile(){
	QString fileName;
	fileName = KFileDialog::getOpenFileName(NULL, "*.mp3 *.ogg", 0, "Open audio file");
	QFileInfo file( fileName );
	if( file.exists() && !file.isDir() ){
		openFile(fileName);
		return true;
	}
	return false;
}

void KTagMainWin::openFile(const QString& fp){
	Ktag* mKtag;
	KIconLoader* ic = new KIconLoader("ktag");
	mKtag = new Ktag(this, fp);
	if(mKtag->setFile(fp)){
		kdDebug("MainWin") << "Adding new tab" << endl;
		tabWidget->insertTab( mKtag, "" );
		ktagList.insert(tabWidget->indexOf(mKtag), mKtag);
		tabWidget->setTabIconSet( mKtag, ic->loadIconSet( "sound_mp3", KIcon::Toolbar ) );
		KURL *url = new KURL(mKtag->mAudioFile);
		QString filename = url->fileName(true);
		tabWidget->setTabLabel( mKtag, filename );
		connect(mKtag, SIGNAL(tabClosed(QWidget *)), this, SLOT(closeTab(QWidget *)));
		connect(mKtag, SIGNAL(modifiedTag(QWidget *, bool)), this, SLOT(updateTabLabel(QWidget *, bool)));
		connect(mKtag, SIGNAL(receivedDropEvent (QDropEvent *)), this, SLOT(dropEvent(QDropEvent *)));
		tabWidget->showPage(mKtag);
		mKtag->setAcceptDrops(TRUE);
	}
	mKtag = 0;
	if(tabWidget->count() == 0){
		kdDebug("MainWin") << "no more tabs, exit" << endl;
		exit(0);
	}
}


void KTagMainWin::closeTab( QWidget* w ){
	kdDebug("MainWin") << "Closing tab " << w->name() << endl;
	w->close();
	tabWidget->removePage(w);
	if(tabWidget->count() == 0){
		qApp->quit();
	}
}

void KTagMainWin::updateTabLabel(QWidget*w, bool b){
	if(b){
		tabWidget->setTabColor(w, QColor("blue"));
		return;
	}
	tabWidget->setTabColor(w, QColor("black"));
}

void KTagMainWin::dragMoveEvent( QDragMoveEvent *e){
	e->accept(false);
}

void KTagMainWin::dropEvent( QDropEvent *e){
	kdDebug("Ktag") << "File dropped" << endl;
	if(e->provides("text/uri-list")){
		e->accept( true );
		e->encodedData("text/uri-list");
		QStrList lst;
		QString filename;
		QUriDrag::decode(e, lst);
		for ( uint i = 0; i < lst.count(); ++i ) {
			openFile(QDir::convertSeparators(QUriDrag::uriToLocalFile(lst.at(i))).utf8());
		}
	}
	else{
		e->ignore();
		kdDebug("Ktag") << "Droped file rejected: " << e->format() << endl;
	}
}

void KTagMainWin::config(){
	KIconLoader* ic = new KIconLoader("ktagConfig");
	KtagConfigDlg* confDlg = new KtagConfigDlg(this);
	patternConfig* patDlg = new patternConfig(confDlg->configStack);
	confDlg->iconBox->insertItem(ic->loadIcon( "download", KIcon::Desktop ), 1);
	
	kdDebug("KTag") << "Looking in ktagrc for patterns" << endl;
	KSimpleConfig conf("ktagrc",false);
	conf.setGroup("ktag");
	QStringList patterns =  conf.readListEntry("patterns", ',');
	
	QStringList::Iterator it = patterns.begin();
	while( it!= patterns.end() ){
		patDlg->addPattern(*it);
		it++;
	}
	
	confDlg->configStack->addWidget(patDlg, 1);
	confDlg->configStack->raiseWidget(patDlg);
	
	connect(confDlg->iconBox, SIGNAL(selected(int )), confDlg->configStack, SLOT(raiseWidget(int)));
	
	if(confDlg->exec()){
		kdDebug("Ktag") << "Save config" << endl;
		QStringList patList;
		for(int i = 0; i <= patDlg->patternList->childCount()-1; i++){
			patList << patDlg->patternList->itemAtIndex(i)->text(0);
		}
		conf.writeEntry("patterns", patList.join(","));
		conf.sync();
	}
	else{
		kdDebug("Ktag") << "Do not save config" << endl;
	}
}

#include "ktagmainwin.moc"
